package com.fzm.mall.pay.entity.dto;

import com.fzm.mall.pay.constant.MallResponseError;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;

/**
 * @author wangp
 * @date 2021/3/18 16:18
 * @description 阿里pay数据传输
 * @since JDK 1.8
 */
@Data
@ApiModel(value = "AlipayDTO", description = "阿里支付数据")
public class AlipayDTO {

    @ApiModelProperty("商品标题/交易标题/订单标题/订单关键字等。注意：不可使用特殊字符，如 /，=，& 等。")
    @NotBlank(message = MallResponseError.NULL_PARAM_ERROR)
    private String subject;

    @ApiModelProperty("商户订单号。64 个字符以内的大小，仅支持字母、数字、下划线。需保证该参数在商户端不重复。")
    @NotBlank(message = MallResponseError.NULL_PARAM_ERROR)
    private String outTradeNo;

    @ApiModelProperty("订单总金额，单位为人民币（元），取值范围为 0.01~100000000.00，精确到小数点后两位。")
    @NotBlank(message = MallResponseError.NULL_PARAM_ERROR)
    private String totalAmount;

}
