package com.fzm.mall.server.admin.distribution.model;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fzm.mall.server.admin.distribution.service.ITemplateItemService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 物流中心-运费模板
 * </p>
 *
 * @author wulixian
 * @since 2021-03-04
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("distribution_template")
@ApiModel(value="Template对象", description="物流中心-运费模板")
public class Template implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "id")
    private Long id;

    @ApiModelProperty(value = "模板名称")
    private String templateName;

    @ApiModelProperty(value = "模板id")
    private String templateId;

    @ApiModelProperty(value = "发货地址id")
    private String deliveryAddressId;

    @ApiModelProperty(value = "发货地址")
    @TableField(exist = false)
    private String deliveryAddressName;

    @ApiModelProperty(value = "运费设置（1-阶梯计价 2-固定运费）")
    private Integer billingType;

    @ApiModelProperty(value = "运费金额")
    private BigDecimal freightAmount;

    @ApiModelProperty(value = "计价方式（1-按件数,2-按重量）")
    private Integer pricingType;

    @ApiModelProperty(value = "默认运费-基数（件或KG）")
    private BigDecimal baseNumber;

    @ApiModelProperty(value = "默认运费-基数对应的费用")
    private BigDecimal baseAmount;

    @ApiModelProperty(value = "默认运费-续件数（件或KG）")
    private BigDecimal addNumber;

    @ApiModelProperty(value = "默认运费-续件单价")
    private BigDecimal addCost;

    @ApiModelProperty(value = "商户id")
    private String merchantId;

    @ApiModelProperty(value = "信息入库时间")
    @TableField(fill = FieldFill.INSERT)
    private Long createTime;

    @ApiModelProperty(value = "信息更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Long updateTime;

    /**
     * 运费规则(非默认)
     */
    @TableField(exist = false)
    private List<TemplateItem> templateItemList;

    /**
     * 批量保存-运费规则(非默认)
     * @param templateItemService
     * @param template
     */
    public void saveBatchItem(ITemplateItemService templateItemService, Template template){
        List<TemplateItem> templateItemList = template.getTemplateItemList();
        templateItemList.stream().forEach(item -> item.init(template));
        templateItemService.saveBatch(templateItemList);
    }

    /**
     * 更新运费模板下的运费规则(非默认)
     * @param templateItemService
     * @param template
     */
    public void updateItem(ITemplateItemService templateItemService, Template template) {
        QueryWrapper<TemplateItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("template_id", template.getTemplateId())
                .eq("merchant_id", template.getMerchantId());
        templateItemService.remove(queryWrapper);

        List<TemplateItem> templateItemList = template.getTemplateItemList();
        templateItemList.stream().forEach(item -> item.init(template));
        templateItemService.saveBatch(templateItemList);
    }

    /**
     * 获取费用模板下的运费规则(非默认)
     * @param templateItemService
     * @param templateId
     * @param merchantId
     */
    public void listItem(ITemplateItemService templateItemService, String templateId, String merchantId) {
        QueryWrapper<TemplateItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("template_id", templateId)
                .eq("merchant_id", merchantId);
        List<TemplateItem> itemList = templateItemService.list(queryWrapper);
        setTemplateItemList(itemList);
    }
}
